package com.yc.ship.module.otc.controller.admin.storeaudit.vo;

import com.yc.ship.framework.common.pojo.PageParam;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.time.LocalDateTime;

import static com.yc.ship.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Schema(description = "管理后台 - 旅行社门店授信提额/授信充值/余额充值分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class StoreAuditPageReqVO extends PageParam {

    @Schema(description = "门店ID", example = "28")
    private Long storeId;

    @Schema(description = "类型 1-1 1-2.余额，2-1 2-2.授信充正，3-1 3-2.授信提额", example = "2")
    private String type;

    @Schema(description = "操作类型 1.余额，2.授信充正，3.授信提额", example = "2")
    private Integer operateType;

    @Schema(description = "变更类型 1.增加、2.减少", example = "2")
    private Integer operateMode;

    @Schema(description = "变更类型 1.增加、2.减少")
    private BigDecimal amount;

    @Schema(description = "申请人")
    private Long applyUser;

    @Schema(description = "申请用户名", example = "王五")
    private String applyUserName;

    @Schema(description = "申请时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] applyTime;

    @Schema(description = "状态 0.申请中，1.审核通过，2.审核驳回", example = "2")
    private Integer status;

    @Schema(description = "审核人")
    private Long auditUser;

    @Schema(description = "审核用户名", example = "王五")
    private String auditUserName;

    @Schema(description = "审核时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] auditTime;

    @Schema(description = "充值备注", example = "你猜")
    private String applyRemark;

    @Schema(description = "审核备注", example = "你说的对")
    private String auditRemark;

    @Schema(description = "充值流水号", example = "8841")
    private String serialNum;

    @Schema(description = "充值单号ID", example = "8841")
    private Long rechargeLogId;

    @Schema(description = "附件地址", example = "https://www.iocoder.cn")
    private String fileUrl;

    @Schema(description = "创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] createTime;

    @Schema(description = "平台租户ID")
    private Long platTenantId;

    @Schema(description = "分销商ID", example = "28")
    private Long distributorId;
}
