package com.yc.ship.module.resource.dal.mysql.room;

import java.util.*;

import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.framework.mybatis.core.query.LambdaQueryWrapperX;
import com.yc.ship.framework.mybatis.core.mapper.BaseMapperX;
import com.yc.ship.module.resource.dal.dataobject.room.ResourceRoomDO;
import org.apache.ibatis.annotations.Mapper;
import com.yc.ship.module.resource.controller.admin.room.vo.*;

/**
 * 资源管理-游轮房间管理 Mapper
 *
 * @author flycloud
 */
@Mapper
public interface ResourceRoomMapper extends BaseMapperX<ResourceRoomDO> {

    default PageResult<ResourceRoomDO> selectPage(ResourceRoomPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<ResourceRoomDO>()
                .eqIfPresent(ResourceRoomDO::getRoomNum, reqVO.getRoomNum())
                .eqIfPresent(ResourceRoomDO::getRoomModelId, reqVO.getRoomModelId())
                .eqIfPresent(ResourceRoomDO::getShipId, reqVO.getShipId())
                .eqIfPresent(ResourceRoomDO::getFloors, reqVO.getFloors())
                .eqIfPresent(ResourceRoomDO::getStatus, reqVO.getStatus())
                .orderByDesc(ResourceRoomDO::getId));
    }

    default List<ResourceRoomDO> selectByShipId(String shipId) {
        return selectList(new LambdaQueryWrapperX<ResourceRoomDO>()
                .eqIfPresent(ResourceRoomDO::getShipId, shipId));
    }

    default ResourceRoomDO selectByShipIdAndRoomNums(Integer roomNum, Long shipId) {
        return selectOne(new LambdaQueryWrapperX<ResourceRoomDO>()
                .eqIfPresent(ResourceRoomDO::getRoomNum, roomNum)
                .eqIfPresent(ResourceRoomDO::getShipId, shipId));
    }

    default List<ResourceRoomDO> selectByShipIdAndFloors(String shipId, List<Integer> floorList) {
        return selectList(new LambdaQueryWrapperX<ResourceRoomDO>()
                .eqIfPresent(ResourceRoomDO::getShipId, shipId)
                .inIfPresent(ResourceRoomDO::getFloors, floorList));
    }
}