package com.yc.ship.module.resource.service.supplier;

import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.module.resource.controller.admin.supplier.vo.*;
import com.yc.ship.module.resource.dal.dataobject.supplier.SupplierDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 资源供应商 Service 接口
 *
 * @author 管理员
 */
public interface SupplierService {

    /**
     * 创建资源供应商
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSupplier(@Valid SupplierSaveReqVO createReqVO);

    /**
     * 更新资源供应商
     *
     * @param updateReqVO 更新信息
     */
    void updateSupplier(@Valid SupplierSaveReqVO updateReqVO);

    /**
     * 删除资源供应商
     *
     * @param id 编号
     */
    void deleteSupplier(Long id);

    /**
     * 获得资源供应商
     *
     * @param id 编号
     * @return 资源供应商
     */
    SupplierDO getSupplier(Long id);
    SupplierRespVO getSupplierById(Long id);

    /**
     * 获得资源供应商分页
     *
     * @param pageReqVO 分页查询
     * @return 资源供应商分页
     */
    PageResult<SupplierRespVO> getSupplierPage(SupplierPageReqVO pageReqVO);

    void updateUserStatus(Long id, Integer status);


    SupplierImportRespVO importSupplierList(List<SupplierImportExcelVO> list, Boolean updateSupport);

    List<SupplierDO> getSimpleListByCategory(SupplierListReqVO reqVO);

    SupplierRespVO getSupplierByDirectAccount(String directAccount);
}